from pysqlsync.formation.object_types import Column, ObjectFactory, Table


class OracleColumn(Column):
    @property
    def data_spec(self) -> str:
        default = f" DEFAULT {self.default}" if self.default is not None else ""
        nullable = " NOT NULL" if not self.nullable and not self.identity else ""
        identity = " GENERATED BY DEFAULT AS IDENTITY" if self.identity else ""
        return f"{self.data_type}{default}{nullable}{identity}"


class OracleTable(Table):
    def alter_table_stmt(self, statements: list[str]) -> str:
        return "\n".join(
            f"ALTER TABLE {self.name} {statement};" for statement in statements
        )

    def drop_if_exists_stmt(self) -> str:
        return (
            "BEGIN\n"
            f"    EXECUTE IMMEDIATE 'DROP TABLE {self.name} CASCADE CONSTRAINTS PURGE';\n"
            "EXCEPTION WHEN OTHERS THEN IF SQLCODE != -942 THEN RAISE; END IF;\n"
            "END;"
        )


class OracleObjectFactory(ObjectFactory):
    @property
    def column_class(self) -> type[Column]:
        return OracleColumn

    @property
    def table_class(self) -> type[Table]:
        return OracleTable
